/* SmashFace App by Hal Nesbitt */

//NOTE: eventually want to cache the returned json in local storage, but need an identifier for the returned photo to match up. Revisit for next version.



//debug javascript
window.onerror = function(message, url, lineNumber) {
    console.log("Error: "+message+" in "+url+" at line "+lineNumber);
}

//declare the global vars
var pictureSource;   // picture source
var destinationType; // sets the format of returned value

//detect if they are dragging an object
var dragging = false;
var dragging_start_y;
var dragging_current_y;

//object holders for the face areas
var nose_tl = {} //4653056 - NOSE_T_L
var nose_tr = {}; //5308416 - NOSE_T_R
var nose_bl = {}; //4849664 - NOSE_BO_NOSTRIL_L
var nose_br = {}; //5111808 - NOSE_BO_NOSTRIL_R

var eye_left_o = {}; //2555904 - EYE_O_L
var eye_left_i = {}; //2818048 - EYE_I_L
var eye_left_t = {}; //2686976 - EYE_T_L
var eye_left_b = {}; //2949120 - EYE_B_L

var eye_right_o = {}; //2031616 - EYE_O_R
var eye_right_i = {}; //2293760 - EYE_I_R
var eye_right_t = {}; //2424832 - EYE_T_R
var eye_right_b = {}; //2162688 - EYE_B_R

var mouth_l = {}; //4128768 - MOUTH_L
var mouth_r = {}; //4390912 - MOUTH_R
var mouth_t = {}; //4259840 - MOUTH_T
var mouth_b = {}; //4521984 - MOUTH_B

var temple_l = {};
var temple_r = {};
var chin_b = {};

var face_tl = {}; //1703936 - TEMPLE_L
var face_tr = {}; //1179648 - TEMPLE_R
var face_bl = {}; //1703936 - TEMPLE_L, 458752 - CHIN_B
var face_br = {}; //1179648 - TEMPLE_R, 458752 - CHIN_B

//misc variables
var edit_mode = "N";

// Wait for Cordova to connect with the device
//
document.addEventListener("deviceready",onDeviceReady,false);

// Cordova is ready to be used!
//

//additional listeners
document.addEventListener("offline", onOffline, false);

function onDeviceReady() {

    //splashscreen!
    navigator.splashscreen.hide();
    
    //assign the image vars
    pictureSource = navigator.camera.PictureSourceType;
    destinationType = navigator.camera.DestinationType;
    
    //bind the load photo button
    $('#load_photo').on('click', function(e) {
                       getPhoto(pictureSource.PHOTOLIBRARY);
    });
    
    //bind the screenshot button
    $('#take_screenshot').on('click', function(e) {
                        takeScreenshot();
    });
    
    //setting the click event listener for the image
    $('#thumb').on('click', function (e) {
                   var offset = $(this).offset();
                   var click_x = e.pageX - offset.left;
                   var click_y = e.pageY - offset.top;
                   checkHit(click_x,click_y);
    });
    
    //bind the edit smash button
    $('#edit_smash').on('click', function (e) {
                    if(!$("#thumb").hasClass("loaded")) {
                        mb.alert('No Photo Selected','You need to load a photo before entering edit mode.');
                        return false;
                    }

                    if(edit_mode == "N") {
                        $("#edit_mode_message").slideDown();
                        removeToolbars();
                        editSmash();
                    } else {
                        killEditMode();
                        removeToolbars();
                        removeAdjust();
                    }
    });
    
    //bind the delete button
    $('#delete_images').on('click', function (e) {
                           dbDeleteImages();
                   });
    
    //bind the photo drag to show/hide the header
    $("#thumb").on('touchstart', function(e) {
                   dragging_start_y = e.originalEvent.touches[0].pageY;
    });
    
    $("#thumb").on('touchmove', function(e) {
                   dragging = true;
                   dragging_current_y = e.originalEvent.touches[0].pageY;
    });
    
    $("#thumb").on('touchend', function(e) {
                   if(dragging && (dragging_current_y - dragging_start_y > 60)) {
                        toggleToolbars();
                   }
                   
                   //reset
                    dragging_start_y = 0;
                    dragging_current_y = 0;
                    dragging = false;
    });
    
    //create a database object to store the img_uids
    var db = window.openDatabase("Database", "1.0", "SmashFace", 200000);
    db.transaction(dbPopulate, fail, dbCreated);
    
    //show the start message
    mb.alert('SmashFace','Welcome to SmashFace! Load a photo and "punch" it to beat it up. To see the controls after you load a photo, just drag the screen down.');

}

//all of the functions used
function onOffline() {
    mb.alert('Internet Required','This application requires an internet connection to function properly.');
}

function takeScreenshot() {
    if($("#thumb").hasClass("loaded")) {
        
        //if we are in edit mode, then kill it
        if(edit_mode != "N") {
            removeAdjust();
            $("#edit_mode_message").hide(); //do a hide to make sure it gets out of the way ASAP
            killEditMode();
        }
        
        //hide some UI components
        $("#header").hide('fast', function(){
                    var height = $("#thumb").height();
                    $("#photo_controls").hide('fast', function() {
                          window.plugins.screenshot.saveScreenshot(height);
                    });
        });
    } else {
        mb.alert('No Photo Selected','Please load a photo before saving.');
    }
}

function getPhoto(source) {
    // Retrieve image file location from specified source
    navigator.camera.getPicture(uploadPhoto, fail, { quality: 50,
                                destinationType: destinationType.FILE_URI,
                                sourceType: source,
                                correctOrientation: true });
}

function uploadPhoto(imageURI) {

    window.plugins.waitingDialog.show("Processing image...");
    
    //reset the smashes and the shake listener
    $(".smash").remove();
    shake.stopWatch();
    
    //set the photo
    var w = $(window).width();
    $("#thumb").hide();
    $("#thumb").attr("src",imageURI).css("width",w + "px").addClass("loaded").show();
    
    //upload this mother
    var options = new FileUploadOptions();
    options.fileKey="qqfile";
    options.fileName=imageURI.substr(imageURI.lastIndexOf('/')+1)+'.png';
    options.mimeType="text/plain";
    
    var params = new Object();
    
    options.params = params;
    
    var ft = new FileTransfer();
    ft.upload(imageURI, encodeURI("http://www.halnesbitt.com/apps/smashface/process-image.php?type=ID&w=" + w), fileUploaded, fail, options);
}

function fileUploaded(r) {
    
    window.plugins.waitingDialog.show("Analyzing face...");
    
    console.log("Code = " + r.responseCode);
    console.log("Response = " + r.response);
    console.log("Sent = " + r.bytesSent);
    var obj = $.parseJSON(r.response);
    console.log("IMG_UID = " + obj.img_uid);
    $.ajax({
           dataType: "jsonp",
           timeout: 30000,
           url: 'http://www.halnesbitt.com/apps/smashface/process-image.php?type=imageinfo&img_uid=' + obj.img_uid,
           jsonpCallback: "defineFacePoints",
           error: ajaxError
    });
    
}

function defineFacePoints(data) {
    
    //check for an empty face object
    if($.isEmptyObject(data.faces)) {
        window.plugins.waitingDialog.hide();
        resetEverything();
        mb.alert('Detection Failed','No face detected or face has been manually deleted.');
        return false;
    }

    //save the faceID to the database
    dbSaveID(data.faces.FaceInfo.uid);
    
    //see if the json is malformed
    if(data.faces.FaceInfo.points == undefined) {
        window.plugins.waitingDialog.hide();
        mb.alert('Undefined Points','The face has undefined points and could not be detected. Please load another photo.');
        return false;
    }

    
	$.each(data.faces.FaceInfo.points.PointInfo, function(i,o) {

		//the nose coordinates
		if(o.type == "4653056") {
			nose_tl[0] = o.x;
			nose_tl[1] = o.y;
		}

		if(o.type == "5308416") {
			nose_tr[0] = o.x;
			nose_tr[1] = o.y;
		}

		if(o.type == "4849664") {
			nose_bl[0] = o.x;
			nose_bl[1] = o.y;
		}

		if(o.type == "5111808") {
			nose_br[0] = o.x;
			nose_br[1] = o.y;
		}

           
		//the left eye coordinates
		if(o.type == "2555904") {
			eye_left_o[0] = o.x;
			eye_left_o[1] = o.y;
		}

		if(o.type == "2818048") {
			eye_left_i[0] = o.x;
			eye_left_i[1] = o.y;
		}

		if(o.type == "2686976") {
			eye_left_t[0] = o.x;
			eye_left_t[1] = o.y;
		}

		if(o.type == "2949120") {
			eye_left_b[0] = o.x;
			eye_left_b[1] = o.y;
		}


		//the right eye coordinates
		if(o.type == "2031616") {
			eye_right_o[0] = o.x;
			eye_right_o[1] = o.y;
		}

		if(o.type == "2293760") {
			eye_right_i[0] = o.x;
			eye_right_i[1] = o.y;
		}

		if(o.type == "2424832") {
			eye_right_t[0] = o.x;
			eye_right_t[1] = o.y;
		}

		if(o.type == "2162688") {
			eye_right_b[0] = o.x;
			eye_right_b[1] = o.y;
		}


		//the mouth coordinates
		if(o.type == "4128768") {
			mouth_l[0] = o.x;
			mouth_l[1] = o.y;
		}

		if(o.type == "4390912") {
			mouth_r[0] = o.x;
			mouth_r[1] = o.y;
		}

		if(o.type == "4259840") {
			mouth_t[0] = o.x;
			mouth_t[1] = o.y;
		}

		if(o.type == "4521984") {
			mouth_b[0] = o.x;
			mouth_b[1] = o.y;
		}


        //the overall face coordinates - grabbing the temples and the chin
        if(o.type == "1703936") {
           temple_l[0] = o.x;
           temple_l[1] = o.y;
        }
           
        if(o.type == "1179648") {
           temple_r[0] = o.x;
           temple_r[1] = o.y;
        }
           
        if(o.type == "458752") {
           chin_b[0] = o.x;
           chin_b[1] = o.y;
        }

	});

    
    //assign the overall face from the temple and chin
    face_tl[0] = temple_l[0];
    face_tl[1] = temple_l[1];
    face_tr[0] = temple_r[0];
    face_tr[1] = temple_r[1];
    face_bl[0] = temple_l[0];
    face_bl[1] = chin_b[1];
    face_br[0] = temple_r[0];
    face_br[1] = chin_b[1];

    //good to go - handle some UI stuff
    window.plugins.waitingDialog.show("Smash away!");
    window.plugins.waitingDialog.hide();
    removeToolbars();
    
    //listen for the shake to reset
    shake.startWatch(resetEverything);
}

function checkHit(x,y) {
    
    //unbind the slider events
    removeAdjust();
    
	//check the nose
	if((x > nose_bl[0] && x < nose_br[0]) && (y > nose_tl[1] && y < nose_bl[1])) {
        var nose = $('<img src="img/smashes/nose_bruise_1.png" class="smash" />');
        var nose_width = nose_br[0] - nose_bl[0];
        var y_adj_ns = parseInt(nose_tl[1] + (nose_tl[1] - nose_bl[1]));
        var x_adj_ns = nose_bl[0];
        nose.css({"width":nose_width,"top":y_adj_ns + "px","left":x_adj_ns + "px","display":"none"});
		$("#thumb_holder").append(nose);
        nose.fadeIn('slow');
        finishSmash();
        return false;
	}

	//check the left eye
	if((x > eye_left_o[0] && x < eye_left_i[0]) && (y > eye_left_t[1] && y < eye_left_b[1])) {
        var lefteye = $('<img src="img/smashes/lefteye_bruise_1.png" class="smash" />');
        var lefteye_width = (eye_left_i[0] - eye_left_o[0])*2; //for some reason, it shows up tiny
        var lefteye_height = (299/348) * lefteye_width; //get the height from the ratio of the original image size
        var y_adj_le = eye_left_o[1] - (lefteye_height/2.5);
        var x_adj_le = eye_left_o[0] - (lefteye_width/4.6);
        lefteye.css({"width":lefteye_width,"top":y_adj_le + "px","left":x_adj_le + "px","display":"none"});
		$("#thumb_holder").append(lefteye);
        lefteye.fadeIn('slow');
        finishSmash();
        return false;
	}

	//check the right eye
	if((x > eye_right_i[0] && x < eye_right_o[0]) && (y > eye_right_t[1] && y < eye_right_b[1])) {
        var righteye = $('<img src="img/smashes/righteye_bruise_1.png" class="smash" />');
        var righteye_width = eye_right_o[0] - eye_left_i[0];
        var righteye_height = (299/348) * righteye_width; //get the height from the ratio of the original image size
        var y_adj_re = eye_right_t[1] - (righteye_height/3.3);
        var x_adj_re = eye_right_i[0] - (righteye_width/3.3);
        righteye.css({"width":righteye_width,"top":y_adj_re + "px","left":x_adj_re + "px","display":"none"});
		$("#thumb_holder").append(righteye);
        righteye.fadeIn('slow');
        finishSmash();
        return false;
	}

	//check the mouth
	if((x > mouth_l[0] && x < mouth_r[0]) && (y > mouth_t[1] && y < mouth_b[1])) {
        var mouth = $('<img src="img/smashes/mouth_bruise_1.png" class="smash" />');
        var mouth_width = mouth_r[0] - mouth_l[0];
        var y_adj_m = mouth_t[1] - 0;
        var x_adj_m = mouth_l[0] - 0;
        mouth.css({"width":mouth_width,"top":y_adj_m + "px","left":x_adj_m + "px","display":"none"});
		$("#thumb_holder").append(mouth);
        mouth.fadeIn('slow');
        finishSmash();
        return false;
	}
    
    //handle the random hits on the face
    if((x > face_tl[0] && x < face_tr[0]) && (y > face_tl[1] && y < face_bl[1])) {
            
        //get random blood image
        var rand_img = getRandomInt(1,3);
            
        var splatter = $('<img src="img/smashes/blood_splatter_' + rand_img + '.png" class="smash" />');
        var splatter_width = face_tr[0] - face_tl[0];
            
        //want it to be random on size, opacity, and rotation
        var rand_size = getRandomDec(0.1,0.5) * splatter_width;
        var rand_opacity = getRandomDec(0.3,0.9);
        var rand_rotate = getRandomInt(0,360);
        var x_blood = x;
        var y_blood = y;
            
        //check to make sure the size is not outside the bounds
        if(x + rand_size > face_tr[0]) {
            x_blood = x - rand_size;
        }
            
        if(y + rand_size > face_br[1]) {
            y_blood = y - rand_size;
        }
            
        splatter.css({"width":rand_size,"top":y_blood + "px","left":x_blood + "px"});
        //splatter.css("opacity",rand_opacity);
        splatter.css("-webkit-transform","rotate(" + rand_rotate + "deg)");
        $("#thumb_holder").append(splatter);
        splatter.fadeTo('slow',rand_opacity);
        finishSmash();
        return false;
    }

    
}

//fiddling with smash functions
function finishSmash() {
    
    //vibrate it
    navigator.notification.vibrate(500);
    
    //bind the longhold to the smashes to bring up the slider menus
    $(".smash").on('taphold', adjustSmash);
    
    //if we are in edit mode, then automatically bind the drag
    if(edit_mode == "Y") {
        dragSmash();
    }
}

function editSmash() {
    edit_mode = "Y";
    dragSmash();
}

function dragSmash() {
    $(".smash").draggable({
        stop: function(event, ui) {
            //if the item is outside the bounds, then remove it
            if(ui.position.top < face_tl[1] || ui.position.top > face_bl[1]) {
                $(this).remove();
            }   
            if(ui.position.left < face_tl[0] || ui.position.left > face_tr[0]) {
                $(this).remove();
            }
        }
    });
}

function adjustSmash(obj){
    if(edit_mode == "Y") {
        
        $("#adjust_sliders").fadeIn('fast');
        
        $(this).addClass("redbox");
        var $smash = $(this);
    
        //adjust the opacity
        $("#slider-0").attr("value", ($smash.css("opacity")*100));
        $("#slider-0").slider('refresh');
        $("#slider-0").bind("change", function(event, ui) {
                            var new_opacity = $(this).val() / 100;
                            $smash.css("opacity",new_opacity);
                            });
        
        //adjust the width (size)
        $("#slider-1").attr("max", $(window).width());
        $("#slider-1").attr("value", $smash.width());
        $("#slider-1").slider('refresh');
        $("#slider-1").bind("change", function(event, ui) {
                            var new_width = $(this).val();
                            $smash.css("width",new_width + "px");
                            });
        
        //adjust the rotation
        $("#slider-2").bind("change", function(event, ui) {
                            var new_rotate = $(this).val();
                            $smash.css("-webkit-transform","rotate(" + new_rotate + "deg)");
                            });
    }
}


//fiddling with UI functions
function toggleToolbars() {
    $("#header").slideToggle();
    $("#photo_controls").slideToggle();
}

function showToolbars() {
    $("#header").slideDown();
    $("#photo_controls").slideDown();
}

function removeToolbars() {
    $("#header").slideUp();
    $("#photo_controls").slideUp();
}

function removeAdjust() {
    $("#adjust_sliders").fadeOut('fast');
    $(".smash").removeClass("redbox");
    $('#slider-0').unbind('change');
    $('#slider-1').unbind('change');
    $('#slider-2').unbind('change');
}

function killEditMode() {
    edit_mode = "N";
    $(".smash").draggable("destroy");
    $("#edit_mode_message").slideUp();
}

function resetEverything() {
    //set everything back to the beginning
    $(".smash").remove();
    $("#thumb").attr("src","img/logo.png").removeClass("loaded");
    removeAdjust();
    killEditMode();
    showToolbars();
    shake.stopWatch();
}

//database functions
function dbPopulate(tx) {
    //when not in debug, comment the drop line to keep the db intact between uses
    //tx.executeSql('DROP TABLE IF EXISTS IMAGES');
    tx.executeSql('CREATE TABLE IF NOT EXISTS IMAGES (id INTEGER PRIMARY KEY, img_uid)');
}

function dbSaveID(img_uid) {
    var db = window.openDatabase("Database", "1.0", "SmashFace", 200000);
    db.transaction(function(tx){
                   tx.executeSql('INSERT INTO IMAGES (img_uid) VALUES ("' + img_uid + '")');
                   }, fail);
}

function dbCreated() {
    //do nothing
    //var db = window.openDatabase("Database", "1.0", "SmashFace", 200000);
    //db.transaction(queryDB, fail);
}

function dbDeleteImages() {
    
    var delete_text = "If you delete an image, you will no longer be able to use it with this app until the binary has been cleared from the Betaface API server cache.";
    
    //stupid bug where the spinner was off after confirm, so show it first
    window.plugins.waitingDialog.show('loading...');
    window.plugins.messageBox.confirm({title: 'Are You Sure?', message: delete_text}, function(button){
                                if(button=="yes") {
                                      var db = window.openDatabase("Database", "1.0", "SmashFace", 200000);
                                      db.transaction(function(tx){
                                                     tx.executeSql('SELECT * FROM IMAGES', [], dbSendDelete, fail);
                                                     }, fail);
                                } else {
                                      window.plugins.waitingDialog.hide();
                                }
                        });

}

function dbSendDelete(tx, results) {
    var len = results.rows.length;
    if(len == 0) {
        window.plugins.waitingDialog.hide();
        mb.alert('No Faces Saved','There is nothing to delete from the server.');
    } else {
        console.log("Number of rows = " + results.rows.length);
        //push the ids into an array
        var faceid_arr = [];
        for (var i=0; i<len; i++){
            console.log("Row = " + i + " ID = " + results.rows.item(i).id + " Data =  " + results.rows.item(i).img_uid);
            faceid_arr.push(results.rows.item(i).img_uid);
        }
        
        //now join the array into a string and ship it off to the server
        var all_images = faceid_arr.join();
        var num_images;
        if(len == 1) {
            num_images = "Deleting 1 image...";
        } else {
            num_images = "Deleting " + len + " images...";
        }
        
        window.plugins.waitingDialog.show(num_images);
        
        $.ajax({
               dataType: "jsonp",
               timeout: 30000,
               url: 'http://www.halnesbitt.com/apps/smashface/process-image.php?type=delete&img_uid=' + all_images,
               jsonpCallback: "dbDeleteFinish",
               error: ajaxError
        });
    
    }
}

function dbDeleteFinish(data){
    //delete the images from the local database by just dropping the whole table and recreating
    var db = window.openDatabase("Database", "1.0", "SmashFace", 200000);
    db.transaction(function(tx){
                   tx.executeSql('DROP TABLE IF EXISTS IMAGES');
                   tx.executeSql('CREATE TABLE IF NOT EXISTS IMAGES (id INTEGER PRIMARY KEY, img_uid)');
                   }, fail);
    
    window.plugins.waitingDialog.show("Images deleted!");
    window.plugins.waitingDialog.hide();
}


//misc return functions
function getRandomDec(min, max) {
    return Math.random() * (max - min) + min;
}

function getRandomInt(min, max) {
    return Math.floor(Math.random() * (max - min + 1)) + min;
}


//the error functions
function fail(error) {
    //if they cancelled
    if(error == "no image selected") {
        return false;
    }
    window.plugins.waitingDialog.hide();
    mb.alert("An error has occurred: Code = " + error.code);
    resetEverything();
    console.log("error message: " + error);
    console.log("upload error source: " + error.source);
    console.log("upload error target: " + error.target);
}

function ajaxError(x,t,m) {
    window.plugins.waitingDialog.hide();
    resetEverything();
    if(t=="timeout") {
        mb.alert('Processing Error','The web service has timed out. Please check your internet settings and try again.');
    } else {
        mb.alert('Processing Error','There was an error completing your request. Please try again.');
    }
}